@@ -94,7 +94,7 @@ def synthesize_midi(synthesis_generator, expression_generator, midi_file,
                     sf2_path='/usr/share/sounds/sf2/FluidR3_GM.sf2',
                     display_progressbar=True,
                     skip_existing_files=False,
-                    save_synth_params=False):
+                    save_metadata=False):
   """
   Synthesize a midi file using MIDI-DDSP.
   Args:
@@ -111,7 +111,8 @@ def synthesize_midi(synthesis_generator, expression_generator, midi_file,
       display_progressbar: Whether to display progress bar.
       skip_existing_files: Skip synthesizing MIDI files if already exist
         output folders.
-      save_synth_params: also save synthesis parameters.
+      save_metadata: also save metadata containing instrument_id, note
+        expression controls, and synthesis parameters.
 
   Returns: A dict of output:
           'mix_audio': mix audio,
@@ -180,12 +181,20 @@ def synthesize_midi(synthesis_generator, expression_generator, midi_file,
         display_progressbar=display_progressbar)
 
     midi_synth_params = midi_synth_params['inputs']
+    conditioning_df_all_for_save = {}
+    instrument_id_all_for_save = {}
     for i in range(midi_audio.shape[0]):
       part_number = part_synth_by_model[i]
 
       # align audio with part number
       midi_audio_all[part_number] = midi_audio[i].numpy()
 
+      # align instrument_id with part number
+      instrument_id_all_for_save[part_number] = instrument_id_all[i].numpy()[0]
+
+      # align note expression controls with part number
+      conditioning_df_all_for_save[part_number] = conditioning_df_all[i]
+
       # align synthesis parameters with part number
       # get the midi synth parameters
       midi_synth_params_all[part_number] = {
@@ -213,14 +222,14 @@ def synthesize_midi(synthesis_generator, expression_generator, midi_file,
                  os.path.join(
                    output_dir,
                    f'{part_number}_{instrument_name}.wav'),
-                 16000)
+                 SAMPLE_RATE)
       elif use_fluidsynth:
         audio = midi_audio_all[part_number]
         save_wav(audio,
                  os.path.join(
                    output_dir,
                    f'{part_number}_{instrument_name}_fluidsynth.wav'),
-                 16000)
+                 SAMPLE_RATE)
 
   # If there is audio synthesized, mix the audio and return the output.
   if midi_audio_all:
@@ -230,10 +239,14 @@ def synthesize_midi(synthesis_generator, expression_generator, midi_file,
         axis=-1),
       axis=-1)
     if output_dir is not None:
-      save_wav(midi_audio_mix, os.path.join(output_dir, 'mix.wav'), 16000)
-      if save_synth_params:
-        pickle_dump(midi_synth_params_all,
-                    os.path.join(output_dir, 'synth_params.pickle'))
+      save_wav(midi_audio_mix, os.path.join(output_dir, 'mix.wav'), SAMPLE_RATE)
+      if save_metadata:
+        metadata = {
+            'instrument_id': instrument_id_all_for_save,
+            'note_expression_control': conditioning_df_all_for_save,
+            'synthesis_parameters': midi_synth_params_all,
+        }
+        pickle_dump(metadata, os.path.join(output_dir, 'metadata.pickle'))
     output = {
       'mix_audio': midi_audio_mix,
       'stem_audio': midi_audio_all,
@@ -283,9 +296,11 @@ def main():
                       action='store_true',
                       help='Skip synthesizing MIDI files if already exist '
                            'output folders.')
-  parser.add_argument('--save_synth_params', dest='save_synth_params',
+  parser.add_argument('--save_metadata', dest='save_metadata',
                       action='store_true',
-                      help='Save synthesis parameters generated by MIDI-DDSP.')
+                      help='Save metadata including containing instrument_id, '
+                           'note expression controls, and synthesis '
+                           'parameters generated by MIDI-DDSP.')
 
   args = parser.parse_args()
 
@@ -320,7 +335,7 @@ def main():
         use_fluidsynth=args.use_fluidsynth,
         display_progressbar=False,
         skip_existing_files=args.skip_existing_files,
-        save_synth_params=args.save_synth_params
+        save_metadata=args.save_metadata,
       )
   elif args.midi_path:
     synthesize_midi(
@@ -334,7 +349,7 @@ def main():
       use_fluidsynth=args.use_fluidsynth,
       display_progressbar=True,
       skip_existing_files=args.skip_existing_files,
-      save_synth_params=args.save_synth_params
+      save_metadata=args.save_metadata,
     )
 
 
